#pragma rtGlobals=2		// Use modern global access method.
// Completion rev. 1.0,  2004-2005 Georg Klein
// Simple implementiation for command line completion
// Drawbacks: 
// 1. Will erase clipboard contents
// 2. After completion, the cursor is at the start of the command line instead of the end

Menu "Tools"
	"Completion/F7",/Q, PerformCompletion()	// Change 'F7' to your preferred shortcut
End

// Function GetCmdLine
// Parameters:	none
// Returntype:	string
// Gets the contents of the command line so far
Function/S GetCmdLine()
	DoWindow/H			
	DoIgorMenu "Edit", "Select All"
	DoIgorMenu "Edit", "Copy"
	
	return GetScrapText()
End

// Function GetPossibleCompletions(cmdLine) 
// Parameters:	string
// Returntype:	string
// Returns a list of all Strings, Variables and Waves that start with cmdLine
Function/S GetPossibleCompletions(cmdLine) 
	String cmdLine
	
	String completionsList = WaveList(cmdLine+"*", ";", "" )
	completionsList += StringList(cmdLine+"*", ";")
	completionsList += VariableList(cmdLine+"*",";",6) + VariableList(cmdLine+"*",";",7)
	return completionsList
End

// Function AddCompletion
// Parameters:	string
// Returntype:	none
// Replaces the existing command line with the completion
// unfortunately, this replaces the clipboard contents as well
Function AddCompletion(completion)
	String completion
	
	PutScrapText completion
	DoWindow/H
	DoIgorMenu "Edit","Clear Cmd Buffer"
	DoIgorMenu "Edit", "Paste"

End

// Function SplitString(source,splitBy,separator)
// Parameters:	string, string, string
// Returntype:	string
// Returns a list of strings separated by separator, 
// derived from the source string and separated by splitBy
// Examples:		SplitString("Hello World", " ", ";" ) returns Hello;World
//				SplitString("Hello World", "o", ";" ) returns Hell;W;rld
Function/S SplitString(source,splitBy,separator)
	String source, splitBy, separator
	String destList = ""
	Variable counter, items = ItemsInList(source, splitBy) -1
	for( counter = 0; counter < items; counter += 1 )
		destList = destList + StringFromList( counter, source, splitBy) + separator
	endfor
	return destList + StringFromList( items,  source, splitBy )
End

// Function GetLargestCommonCompletion( cmd, completionsList, separator )
// Parameters:	string, string, string
// Returntype:	string
// Returns a string that consists of all common characters from the completions list
// Examples:	GetLargestCommonCompletion( "Tes", "TestThis;TestThat", ";" ) returns "TestTh"
// 				GetLargestCommonCompletion( "Tes", "TestThis;TestThat;TestAll", ";" ) returns "Test"
// 				GetLargestCommonCompletion( "Te", "TestAll", ";" ) returns "TestAll"
Function/S GetLargestCommonCompletion( cmd, completionsList, separator )
	String cmd, completionsList, separator
	
	Variable counter, length, inner
	Variable items = ItemsInList( completionsList, separator)
	String curSubstr = StringFromList(0,completionsList, separator)
	length = strlen(curSubstr)
	for( counter = strlen(cmd); counter < length; counter += 1 )
		for( inner = 0; inner < items; inner += 1 )
			if( strsearch(StringFromList(inner, completionsList,separator),curSubstr[0,counter],0) == -1)
				return curSubstr[0,counter-1]
			endif
		endfor
	endfor
	return curSubstr
End

// Function PerformCompletion()
// Parameters:	none
// Returntype:	none
// Performs the actual completion by calling the various functions 
// Theory of operation:
//1. Get the current command line
//2. Derive the last word, this will be completed
//3. Create a list of Waves, Variables and Strings starting with the last word in the comand line
//4. if there are no completions, beep, if there is one completion, add it, if there are multiple
//	possible completions, add the common letters of all possible completions
//5. replace the command line with the new one. 
Function PerformCompletion()
	String cmdLine = GetCmdLine()
	String cmdList = SplitString(cmdLine," ",";")
	Variable counter, items = ItemsInList(cmdList,";") - 1
	String completionsList = GetPossibleCompletions(StringFromList(items, cmdList, ";"))
	Variable completions = ItemsInList( completionsList, ";" )
	if( completions != 0 )
		String newCmdLine = StringFromList(0, cmdList,";")
		for( counter = 1; counter < items; counter += 1 )
			newCmdLine = newCmdLine + " " + StringFromList( counter, cmdList, ";" )
		endfor
		if( completions == 1 )
			AddCompletion( newCmdLine + " " + StringFromList( 0, completionsList, ";") )
		else
			AddCompletion( newCmdLine + " " + GetLargestCommonCompletion( StringFromList(items, cmdList, ";"),completionsList, ";") )
		endif
	else
		Beep
	endif
End

